<?php

namespace _PhpScoperAtome\Atome\Payment;

use _PhpScoperAtome\Atome\Enums\Atome;
use _PhpScoperAtome\Atome\Enums\OrderStatus;
use _PhpScoperAtome\Atome\Enums\PaymentStatus;
use _PhpScoperAtome\Atome\Services\Logger;
use _PhpScoperAtome\Atome\Services\Sentry;
use Exception;
use Throwable;
use WC_Order;
/** @internal */
class Callback extends Payment
{
    public function handle()
    {
        $orderId = $_GET['referenceId'] ?? null;
        if (!$orderId) {
            return $this->response(\false, 'Query parameter `referenceId` is required.');
        }
        $apiUrl = Atome::getApiBaseUrl() . 'payments/' . $orderId;
        $response = wp_remote_get($apiUrl, ['method' => 'GET', 'headers' => $this->getHeader()]);
        Logger::instance()->info('callback_handler Get Payment Info Response: ' . \json_encode($response));
        if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200 || !isset($response['body']) || empty($response['body'])) {
            Logger::instance()->error('Failed to get payment information request：' . \json_encode($response), \true);
            return $this->response(\false, \json_encode(['message' => 'HTTP request failed.', 'url' => $apiUrl, 'response' => $response]));
        }
        $body = \json_decode($response['body'], \true);
        if (!isset($body['status'])) {
            return $this->response(\false, 'Response missing `status` data.');
        }
        Logger::instance()->info(__METHOD__ . ':' . $body['status']);
        switch ($body['status']) {
            case PaymentStatus::PAID:
                $this->updateOrderWhenPaid($orderId);
                break;
            case PaymentStatus::CANCELLED:
                $this->updateOrderStatusWhenFailed($orderId, OrderStatus::CANCELLED);
                break;
            case PaymentStatus::FAILED:
                $this->updateOrderStatusWhenFailed($orderId, OrderStatus::FAILED);
                break;
            case PaymentStatus::REFUNDED:
                $this->updateOrderStatusWhenFailed($orderId, OrderStatus::REFUNDED);
                break;
            case PaymentStatus::PROCESSING:
                return $this->response(\false, "Payment is still processing, ignored.");
            default:
                return $this->response(\false, "The `status` parameter is not one of the known statuses: {$body['status']}");
        }
        return $this->response();
    }
    protected function updateOrderStatusWhenFailed($orderId, $orderStatus)
    {
        $order = wc_get_order($orderId);
        if (!$order instanceof WC_Order || $order->get_status() !== OrderStatus::PENDING) {
            return;
        }
        $order->update_status($orderStatus);
        Logger::instance()->info(__METHOD__ . ':' . $orderStatus);
    }
    protected function updateOrderWhenPaid($referenceId)
    {
        global $wpdb;
        try {
            $order = wc_get_order($referenceId);
            if (!$order instanceof WC_Order) {
                throw new Exception("Order({$referenceId}) not found.");
            }
            if (OrderStatus::PENDING !== $order->get_status()) {
                throw new Exception('The order status is not `PENDING`, which is usually due to multiple callbacks occurring: ' . $order->get_status());
            }
            $wpdb->query('START TRANSACTION');
            $order->update_status(OrderStatus::PROCESSING);
            $order->payment_complete();
            wc_reduce_stock_levels($order->get_id());
            $wpdb->query('COMMIT');
            return $this->response();
        } catch (Throwable $e) {
            $wpdb->query('ROLLBACK');
            Logger::instance()->error('Atome Callback Handler Exception: ' . wc_print_r($e, \true));
            Sentry::instance()->capture($e);
            return $this->response(\false, $e->getMessage() . $e->getTraceAsString());
        }
    }
    protected function response($OK = \true, $message = null, $statusCode = null)
    {
        if ($OK) {
            wp_send_json(['code' => 200, 'message' => 'OK']);
        } else {
            wp_send_json(['code' => 500, 'message' => $message], $statusCode ?: 400);
        }
        return null;
    }
}
